"""Contains the processor split class."""


import logging


class ProcessorSplit:
    """
    Class which sets the processor split.

    Attributes
    ----------
    __number_of_processors : None or int
        Getter and setter variable for number_of_processors
    __number_of_nodes : None or int
        Getter and setter variable for number_of_nodes
    __processors_per_node : None or int
        Getter and setter variable for processors_per_node
    number_of_processors : int
        The total number of processors to use
    number_of_nodes : int
        How many nodes to run on (only effective on clusters)
    processors_per_node : int
        The number of processors to allocate per node
        (only effective on clusters)

    Examples
    --------
    >>> processor_split = ProcessorSplit(number_of_processors=1,
    ...                                  number_of_nodes=1,
    ...                                  processors_per_node=1)

    >>> processor_split = ProcessorSplit(number_of_processors=2,
    ...                                  number_of_nodes=1,
    ...                                  processors_per_node=1)
    Traceback (most recent call last):
      File "<input>", line 1, in <module>
    ValueError: number_of_nodes*processors_per_node = 1, whereas
    number_of_processors = 2
    """

    def __init__(
        self,
        number_of_processors: int = 1,
        number_of_nodes: int = 1,
        processors_per_node: int = 1,
    ) -> None:
        """
        Set the parameters.

        Parameters
        ----------
        number_of_processors : int
            The total number of processors to use
        number_of_nodes : int
            How many nodes to run on (only effective on clusters)
        processors_per_node : int
            The number of processors to allocate per node
            (only effective on clusters)
        """
        # Declare variables to be used in the getters and setters
        self.__number_of_processors = number_of_processors
        self.__number_of_nodes = number_of_nodes
        self.__processors_per_node = processors_per_node

        # Set the number of processors
        self.number_of_processors = number_of_processors

        # Set the number of nodes
        self.number_of_nodes = number_of_nodes

        # Set the processors per node
        self.processors_per_node = processors_per_node

    @property
    def number_of_processors(self) -> int:
        """
        Set the properties of self.number_of_processors.

        Returns
        -------
        int
            The number of processors
        """
        return self.__number_of_processors

    @number_of_processors.setter
    def number_of_processors(self, number_of_processors: int):
        self.__number_of_processors = number_of_processors
        if self.number_of_nodes is not None and self.processors_per_node is not None:
            self.__enough_nodes_check()

        logging.debug("number_of_processors set to %s", number_of_processors)

    @property
    def number_of_nodes(self) -> int:
        """
        Set the properties of self.number_of_nodes.

        Returns
        -------
        int
            The number of nodes
        """
        return self.__number_of_nodes

    @number_of_nodes.setter
    def number_of_nodes(self, number_of_nodes: int):
        self.__number_of_nodes = number_of_nodes
        if (
            self.number_of_processors is not None
            and self.processors_per_node is not None
        ):
            self.__enough_nodes_check()
        logging.debug("number_of_nodes set to %s", number_of_nodes)

    @property
    def processors_per_node(self) -> int:
        """
        Set the properties of self.processors_per_node.

        Returns
        -------
        int
            The number of nodes
        """
        return self.__processors_per_node

    @processors_per_node.setter
    def processors_per_node(self, processors_per_node: int):
        self.__processors_per_node = processors_per_node
        if self.number_of_processors is not None and self.number_of_nodes is not None:
            self.__enough_nodes_check()
        logging.debug("processors_per_node set to %s", processors_per_node)

    def __enough_nodes_check(self) -> None:
        """
        Check that enough nodes are allocated.

        Raises
        ------
        ValueError
            If
            self.number_of_nodes * self.processors_per_node < self.number_of_processors
        """
        product = self.number_of_nodes * self.processors_per_node
        if product < self.number_of_processors:
            msg = (
                f"number_of_nodes*processors_per_node = {product}, "
                f"whereas number_of_processors = "
                f"{self.number_of_processors}"
            )
            raise ValueError(msg)



"""Contains unittests for the reading of makefiles."""


from pathlib import Path

import pytest
from bout_runners.make.read_makefile import (
    BoutMakefileReader,
    BoutMakefileVariableReader,
    MakefileReaderError,
)


def test_read_bout_makefile(get_test_data_path: Path) -> None:
    """
    Test that BoutMakefileReader can read a file.

    Parameters
    ----------
    get_test_data_path : Path
        Path to the test data
    """
    reader = BoutMakefileReader(get_test_data_path.joinpath("test_read"))
    assert reader.content == "This is some text\n"


@pytest.mark.parametrize(
    "filename,expected",
    [("Makefile_value", "val 123 val.cxx.foo"), ("Makefile_multiple_value", "not_val")],
)
def test_get_variable_value(
    filename: str, expected: str, get_test_data_path: Path
) -> None:
    """
    Test that get_variable is reading variables properly.

    Parameters
    ----------
    filename : str
        Name of the file to run the test on
    expected : str
        Expected result
    get_test_data_path :  Path
        Path to the test data
    """
    var = BoutMakefileVariableReader(get_test_data_path.joinpath(filename), "VAR")
    val = var.value

    assert val == expected


def test_get_variable_value_raises(get_test_data_path: Path) -> None:
    """
    Test that MakefileReaderError is properly raised.

    Parameters
    ----------
    get_test_data_path :  Path
        Path to the test data
    """
    var = BoutMakefileVariableReader(
        get_test_data_path.joinpath("Makefile_only_comment"), "VAR"
    )

    with pytest.raises(MakefileReaderError):
        _ = var.value



"""Package builder for bout_runners."""

import setuptools

# python setup.py sdist bdist_wheel
# twine upload dist/*
# pip install .[tests,fast]
# or on zsh
# pip install ./[tests,fast/]


# .pypirc
# [distutils]
# index-servers =
#     pypi
#
# [pypi]
# repository: https://upload.pypi.org/legacy/
# username:
# password:
setuptools.setup()



"""Module containing the LogReader class."""


import logging
import re
from datetime import datetime
from pathlib import Path
from typing import Optional


class LogReader:
    """
    Class for reading BOUT++ log files.

    Attributes
    ----------
    file_str : str
        The log file as a string
    start_time : None or str
        The time of the execution start (given that it has started)
    end_time : None or str
        The time of the execution end (given that it has started)
    pid : None or int
        The processor id of the part of the program writing to the
        log file

    Methods
    -------
    started()
        Whether or not the execution has started
    ended()
        Whether or not the execution has ended
    pid_exist()
        Whether or not the pid can be found
    __is_str_in_file(pattern)
        Check whether regex-pattern exists in file
    __find_locale_time(pattern)
        Return the locale time of a regex capture

    Examples
    --------
    >>> from pathlib import Path
    >>> path = Path().joinpath('path', 'to', 'BOUT.log.0')
    >>> log_reader = LogReader(path)
    >>> log_reader.start_time
    '2020-05-01 17:07:10'

    >>> log_reader.end_time
    '2020-05-01 17:07:14'

    >>> log_reader.pid
    1190
    """

    def __init__(self, log_path: Path) -> None:
        """
        Open and read a log file.

        Parameters
        ----------
        log_path : str or Path
            Absolute path to log file
        """
        with Path(log_path).open("r") as log_file:
            self.file_str = log_file.read()
            logging.debug("Opened log_file %s", log_path)

    def started(self) -> bool:
        """
        Check whether the run has a start time.

        Returns
        -------
        bool
            True if the start signature is found in the file
        """
        return self.__is_str_in_file(r"^Run started at")

    def ended(self) -> bool:
        """
        Check whether the run has an end time.

        Returns
        -------
        bool
            True if the end signature is found in the file
        """
        return self.__is_str_in_file(r"^Run finished at")

    def pid_exist(self) -> bool:
        """
        Check whether a process id exist.

        Returns
        -------
        bool
            True if the pid is found
        """
        return self.__is_str_in_file(r"^pid\s*:\s*")

    @property
    def start_time(self) -> Optional[datetime]:
        """
        Return the start time of the process.

        Returns
        -------
        datetime or None
            The start time on date time format
        """
        if self.started():
            return self.__find_locale_time(r"^Run started at  : (.*)")
        return None

    @property
    def end_time(self) -> Optional[datetime]:
        """
        Return the end time of the process.

        Returns
        -------
        datetime or None
            The end time on date time format
        """
        if self.ended():
            return self.__find_locale_time(r"^Run finished at  : (.*)")
        return None

    @property
    def pid(self) -> Optional[int]:
        """
        Return the pid of the process.

        Returns
        -------
        int or None
            The pid of the process
        """
        if self.pid_exist():
            pattern = r"^pid:\s*(\d*)\s*$"
            # Using search as match will only search the beginning of
            # the string
            # https://stackoverflow.com/a/32134461/2786884
            match = re.search(pattern, self.file_str, flags=re.MULTILINE)
            if match is None:
                return None
            return int(match.group(1))
        return None

    def __is_str_in_file(self, pattern: str) -> bool:
        """
        Check whether regex-pattern exists in file.

        Parameters
        ----------
        pattern : str
            Regex pattern to search for

        Returns
        -------
        bool
            True if pattern exist
        """
        # Using search as match will only search the beginning of the
        # string
        # https://stackoverflow.com/a/32134461/2786884
        match = re.search(pattern, self.file_str, flags=re.MULTILINE)
        if match is None:
            return False
        return True

    def __find_locale_time(self, pattern: str) -> datetime:
        """
        Return the locale time of a regex capture.

        Parameters
        ----------
        pattern : str
            String to search for

        Returns
        -------
        time : datetime
            The locale datetime

        Raises
        ------
        ValueError
            If no matches for pattern is found in self.file_str
        """
        # Using search as match will only search the beginning of the
        # string
        # https://stackoverflow.com/a/32134461/2786884
        match = re.search(pattern, self.file_str, flags=re.MULTILINE)
        if match is None:
            raise ValueError(f"No matches in {self.file_str} with pattern {pattern}")

        time_str = match.group(1)
        time = datetime.strptime(time_str, "%c")
        return time



"""Contains the BOUT runner class."""


import logging
from typing import Optional

from bout_runners.database.database_connector import DatabaseConnector
from bout_runners.database.database_creator import DatabaseCreator
from bout_runners.executor.executor import Executor
from bout_runners.metadata.metadata_recorder import MetadataRecorder
from bout_runners.parameters.final_parameters import FinalParameters


class BoutRunner:
    r"""
    Class for executing a run and store its metadata.

    Attributes
    ----------
    self.__executor : Executor
        Getter variable for executor
    self.__db_connector : DatabaseConnector
        Getter variable for db_connector
    self.__final_parameters : FinalParameters
        Getter variable for final_parameters
    self.__db_creator : DatabaseCreator
        Object used to create the database
    self.__metadata_recorder : MetadataRecorder
        Object used to record the metadata about a run
    self.executor : Executor
        Object used to execute the run
    self.db_creator : DatabaseCreator
        Object used to create the database
    self.final_parameters : FinalParameters
        Object containing the parameters to use

    Methods
    -------
    create_schema()
        Create the schema
    run(force)
        Execute the run

    Examples
    --------
    The easiest way to use BoutRunner is to run a script from the root directory of
    the project (i.e. where the `Makefile` and `data` directory are normally
    situated. The script can simply call

    >>> BoutRunner().run()

    and `BoutRunner` takes care of the rest.

    A more elaborate example where all the dependency objects are built manually:

    Import dependencies

    >>> from pathlib import Path
    >>> from bout_runners.executor.bout_paths import BoutPaths
    >>> from bout_runners.executor.executor import Executor
    >>> from bout_runners.database.database_connector import DatabaseConnector
    >>> from bout_runners.parameters.default_parameters import DefaultParameters
    >>> from bout_runners.parameters.run_parameters import RunParameters
    >>> from bout_runners.parameters.final_parameters import FinalParameters
    >>> from bout_runners.submitter.local_submitter import LocalSubmitter

    Create the `bout_paths` object

    >>> project_path = Path().joinpath('path', 'to', 'project')
    >>> bout_inp_src_dir = Path().joinpath('path', 'to', 'source', 'BOUT.inp')
    >>> bout_inp_dst_dir = Path().joinpath('path', 'to', 'destination', 'BOUT.inp')
    >>> bout_paths = BoutPaths(project_path=project_path,
    ...                        bout_inp_src_dir=bout_inp_src_dir,
    ...                        bout_inp_dst_dir=bout_inp_dst_dir)

    Create the input objects

    >>> default_parameters = DefaultParameters(bout_paths)
    >>> run_parameters = RunParameters({'global': {'nout': 0}})
    >>> final_parameters = FinalParameters(default_parameters,
    ...                                    run_parameters)
    >>> executor = Executor(
    ...     bout_paths=bout_paths,
    ...     submitter=LocalSubmitter(bout_paths.project_path),
    ...     run_parameters=run_parameters)
    >>> db_connection = DatabaseConnector('name_of_database', db_root_path=Path())

    Run the project

    >>> runner = BoutRunner(executor,
    ...                     db_connection,
    ...                     final_parameters)
    >>> runner.run()
    """

    def __init__(
        self,
        executor: Optional[Executor] = None,
        db_connector: Optional[DatabaseConnector] = None,
        final_parameters: Optional[FinalParameters] = None,
    ) -> None:
        """
        Set the member data.

        Parameters
        ----------
        executor : Executor or None
            Object executing the run
            If None, default parameters will be used
        db_connector : DatabaseConnector or None
            The connection to the database
            If None: Default database connector will be used
        final_parameters : FinalParameters or None
            The object containing the parameters which are going to be used in the run
            If None, default parameters will be used
        """
        # Set member data
        # NOTE: We are not setting the default as a keyword argument
        #       as this would mess up the paths
        self.__executor = executor if executor is not None else Executor()
        self.__final_parameters = (
            final_parameters if final_parameters is not None else FinalParameters()
        )
        self.__db_connector = (
            db_connector if db_connector is not None else DatabaseConnector()
        )
        self.__db_creator = DatabaseCreator(self.db_connector)
        self.__metadata_recorder = MetadataRecorder(
            self.__db_connector, self.executor.bout_paths, self.final_parameters
        )

    @property
    def executor(self) -> Executor:
        """
        Get the properties of self.executor.

        Returns
        -------
        self.__executor : Executor
            The executor object
        """
        return self.__executor

    @property
    def final_parameters(self) -> FinalParameters:
        """
        Get the properties of self.final_parameters.

        Returns
        -------
        self.__final_parameters : FinalParameters
            The object containing the parameters used in the run
        """
        return self.__final_parameters

    @property
    def db_connector(self) -> DatabaseConnector:
        """
        Get the properties of self.db_connector.

        Returns
        -------
        self.__db_connector : DatabaseConnector
            The object holding the database connection
        """
        return self.__db_connector

    def create_schema(self) -> None:
        """Create the schema."""
        final_parameters_dict = self.final_parameters.get_final_parameters()
        final_parameters_as_sql_types = self.final_parameters.cast_to_sql_type(
            final_parameters_dict
        )
        self.__db_creator.create_all_schema_tables(final_parameters_as_sql_types)

    def run(self, force: bool = False) -> None:
        """
        Perform the run and capture data.

        Parameters
        ----------
        force : bool
            Execute the run even if has been performed with the same parameters
        """
        if not self.__metadata_recorder.db_reader.check_tables_created():
            logging.info(
                "Creating schema as no tables were found in " "%s",
                self.__metadata_recorder.db_reader.db_connector.db_path,
            )
            self.create_schema()

        run_id = self.__metadata_recorder.capture_new_data_from_run(
            self.__executor.submitter.processor_split, force
        )

        if run_id is None:
            logging.info("Executing the run")
            self.executor.execute()
        else:
            logging.warning(
                "Run with the same configuration has been executed before, "
                "see run with run_id %d",
                run_id,
            )
            if force:
                logging.info("Executing the run as force==True")
                self.executor.execute()



"""Contains unittests for the metadata_reader."""


from typing import Dict, List

from bout_runners.metadata.metadata_reader import MetadataReader
from pandas import DataFrame


def test_get_table_column_dict(
    yield_metadata_reader: MetadataReader, yield_all_metadata: DataFrame
) -> None:
    """
    Test that the metadata reader retrieves the table column dict.

    Parameters
    ----------
    yield_metadata_reader : MetadataReader
        The metadata reader object
    yield_all_metadata : DataFrame
        The test metadata
    """
    table_columns_dict = yield_metadata_reader.table_column_dict
    # Extract columns dict
    all_metadata = yield_all_metadata
    expected_columns_dict: Dict[str, List[str]] = dict()
    for pandas_col in all_metadata.columns:
        table, col = pandas_col.split(".")
        if table not in expected_columns_dict.keys():
            expected_columns_dict[table] = list()
        expected_columns_dict[table].append(col)

    # Check that the tables are the same
    columns_keys = sorted(table_columns_dict.keys())
    expected_columns_keys = sorted(expected_columns_dict.keys())
    assert set(columns_keys) == set(expected_columns_keys)

    # Check that the columns are the same
    for column_name, expected_column_name in zip(columns_keys, expected_columns_keys):
        columns = table_columns_dict[column_name]
        expected_columns = expected_columns_dict[expected_column_name]
        assert set(columns) == set(expected_columns)


def test_get_table_connections(yield_metadata_reader: MetadataReader) -> None:
    """
    Test that the metadata reader retrieves the table connections.

    Parameters
    ----------
    yield_metadata_reader : MetadataReader
        The metadata reader object
    """
    table_connections = yield_metadata_reader.table_connection
    expected_connections = {
        "parameters": ("foo", "bar", "baz"),
        "run": ("file_modification", "split", "parameters", "system_info"),
    }
    assert table_connections == expected_connections


def test_get_sorted_columns(
    yield_metadata_reader: MetadataReader, yield_all_metadata: DataFrame
) -> None:
    """
    Test that the metadata reader retrieves sorted columns.

    Parameters
    ----------
    yield_metadata_reader : MetadataReader
        The metadata reader object
    yield_all_metadata : DataFrame
        The test metadata
    """
    sorted_columns = yield_metadata_reader.sorted_columns
    expected = tuple(yield_all_metadata.columns)
    assert sorted_columns == expected


def test_get_parameters_metadata(
    yield_metadata_reader: MetadataReader, yield_all_metadata: DataFrame
) -> None:
    """
    Test that the metadata reader can get the parameters metadata.

    Parameters
    ----------
    yield_metadata_reader : MetadataReader
        The metadata reader object
    yield_all_metadata : DataFrame
        The test metadata
    """
    tables_to_keep = ("bar", "baz", "foo", "parameters")
    cols_to_keep = [
        col for col in yield_all_metadata.columns if col.split(".")[0] in tables_to_keep
    ]
    expected = yield_all_metadata.loc[:, cols_to_keep]
    expected.drop_duplicates(inplace=True)
    metadata_reader = yield_metadata_reader
    parameters_metadata = metadata_reader.get_parameters_metadata()
    assert parameters_metadata.equals(expected)


def test_get_all_metadata(
    yield_metadata_reader: MetadataReader, yield_all_metadata: DataFrame
) -> None:
    """
    Test that the metadata reader and the drop_id decorator works.

    Parameters
    ----------
    yield_metadata_reader : MetadataReader
        The metadata reader object
    yield_all_metadata : DataFrame
        The test metadata
    """
    expected = yield_all_metadata
    metadata_reader = yield_metadata_reader
    all_metadata = metadata_reader.get_all_metadata()
    assert all_metadata.equals(expected)

    metadata_reader.drop_id = "parameters"
    no_parameters_df = metadata_reader.get_all_metadata()
    no_parameter_columns = [
        col for col in tuple(expected.columns) if not col.startswith("parameters.")
    ]
    assert list(no_parameters_df.columns) == no_parameter_columns

    metadata_reader.drop_id = "keep_run_id"
    only_run_id_df = metadata_reader.get_all_metadata()
    only_run_id_columns = [
        col
        for col in no_parameter_columns
        if not ((col.endswith(".id") and not col == "run.id") or col.endswith("_id"))
    ]
    assert list(only_run_id_df.columns) == only_run_id_columns

    metadata_reader.drop_id = "all_id"
    all_id_df = metadata_reader.get_all_metadata()
    all_id = [col for col in only_run_id_columns if not col == "run.id"]
    assert list(all_id_df.columns) == all_id



"""Contains the class dealing with the run parameters."""


import logging
from typing import Dict, Optional, Union


class RunParameters:
    """
    Class which deals with run parameters with precedence over BOUT.inp.

    Attributes
    ----------
    __run_parameters_dict : None or dict
        Getter and setter variable for run_parameters_dict
    __run_parameters_str : None or str
        Getter and setter variable for run_parameters_str
    run_parameters_dict : dict
        The run parameters on dictionary form
    run_parameters_str : None or str
        The run parameters on string form

    Examples
    --------
    >>> run_parameters = RunParameters({'mesh':  {'nx': 4}})
    >>> run_parameters.run_parameters_str
    'mesh.nx=4 '

    >>> run_parameters.run_parameters_str = 'foo'
    Traceback (most recent call last):
      File "<input>", line 1, in <module>
    AttributeError: The run_parameters_str is read only, and is set
    internally in the setter of run_parameters_dict
    """

    def __init__(
        self,
        run_parameters_dict: Optional[
            Dict[str, Dict[str, Union[int, float, bool, str]]]
        ] = None,
    ) -> None:
        """
        Set the parameters.

        Parameters
        ----------
        run_parameters_dict : None or dict
            Options on the form

            >>> {'global': {'append': False, 'nout': 5},
            ...  'mesh':  {'nx': 4},
            ...  'section_in_BOUT_inp': {'some_variable': 'some_value'}}

        Notes
        -----
        The parameters set here will override those found in the BOUT.inp file
        """
        # Declare variables to be used in the getters and setters
        self.__run_parameters_dict = None
        self.__run_parameters_str: Optional[str] = None

        # Set the parameters dict (and create the parameters string)
        self.run_parameters_dict = run_parameters_dict

    @property
    def run_parameters_dict(
        self,
    ) -> Optional[Dict[str, Dict[str, Union[int, float, bool, str]]]]:
        """
        Set the properties of self.run_parameters_dict.

        The setter will also create the self.__run_parameters_str
        The run_parameters_dict should be set on the form

        >>> {'global': {'append': False, 'nout': 5},
        ...  'mesh':  {'nx': 4},
        ...  'section_in_BOUT_inp': {'some_variable': 'some_value'}}

        Returns
        -------
        self.__run_parameters_dict : dict of str, dict
            Absolute path to the root of make file

        Raises
        ------
        RuntimeError
            If self.run_parameters_dict is None
        """
        return self.__run_parameters_dict

    @run_parameters_dict.setter
    def run_parameters_dict(
        self,
        run_parameters_dict: Optional[
            Dict[str, Dict[str, Union[int, float, bool, str]]]
        ],
    ) -> None:
        # Set the run parameters
        self.__run_parameters_dict = (
            run_parameters_dict if run_parameters_dict is not None else dict()
        )

        # Assert to prevent "Incompatible types in assignment" with Optional
        if self.run_parameters_dict is None:
            raise RuntimeError("self.run_parameters_dict is None")

        # Generate the string
        sections = list(self.run_parameters_dict.keys())
        self.__run_parameters_str = ""
        if "global" in sections:
            # Removing global, as this is something added in
            # obtain_project_parameters
            sections.remove("global")
            global_option = self.run_parameters_dict["global"]
            for key, val in global_option.items():
                self.__run_parameters_str += f"{key}={val} "

        for section in sections:
            for key, val in self.__run_parameters_dict[section].items():
                self.__run_parameters_str += f"{section}.{key}={val} "
        logging.debug("Parameters set to %s", self.__run_parameters_str)

    @property
    def run_parameters_str(self) -> str:
        """
        Set the properties of self.run_parameters_str.

        Returns
        -------
        str
            The parameters dict serialized as a string

        Raises
        ------
        RuntimeError
            If self.__run_parameters_str is None

        Notes
        -----
        As the run_parameters_str must reflect run_parameters_dict, both are set when
        setting run_parameters_dict
        """
        # Assert to prevent "Incompatible return type" with Optional
        if self.__run_parameters_str is None:
            raise RuntimeError("self.__run_parameters_str is None")
        return self.__run_parameters_str



"""Contains methods which return common paths."""


import configparser
import time
from configparser import ConfigParser
from pathlib import Path
from typing import Optional


def get_bout_runners_package_path() -> Path:
    """
    Return the absolute path to the bout_runners package.

    Returns
    -------
    Path
        The path to the root directory
    """
    return Path(__file__).absolute().parents[1]


def get_config_path() -> Path:
    """
    Return the absolute path to the configurations.

    Returns
    -------
    Path
        The path to the configuration directory
    """
    return get_bout_runners_package_path().joinpath("config")


def get_logger_config_path() -> Path:
    """
    Return the absolute path to the logger configuration.

    Returns
    -------
    Path
        The path to the logger configuration file
    """
    return get_config_path().joinpath("logging_config.yaml")


def get_bout_runners_config_path() -> Path:
    """
    Return the absolute path to the bout_runners configuration.

    Returns
    -------
    Path
        The path to the bout_runners configuration file
    """
    return get_config_path().joinpath("bout_runners.ini")


def get_bout_log_config_path() -> Path:
    """
    Return the absolute path to the log configuration.

    Returns
    -------
    Path
        The path to the bout_runners configuration file
    """
    return get_config_path().joinpath("logging_config.yaml")


def get_bout_runners_configuration() -> ConfigParser:
    """
    Return the bout_runners configuration.

    Returns
    -------
    config : ConfigParser
        The configuration of bout_runners
    """
    config = configparser.ConfigParser()
    config.read(get_bout_runners_config_path())
    return config


def get_log_file_directory() -> Path:
    """
    Return the log_file directory.

    Returns
    -------
    log_file_directory : Path
        Path to the log_file directory
    """
    config = get_bout_runners_configuration()
    path_str = config["log"]["directory"]
    if path_str == "None":
        log_file_dir = get_bout_runners_package_path().joinpath("logs")
    else:
        log_file_dir = Path(path_str)

    log_file_dir.mkdir(exist_ok=True, parents=True)
    return log_file_dir


def get_log_file_path(
    log_file_dir: Optional[Path] = None, name: Optional[str] = None
) -> Path:
    """
    Return the absolute path to the log file path.

    Parameters
    ----------
    log_file_dir : Path or None
        Path to the log file directory
        If None, default log file directory will be used
    name : str or None
        Name of the log file
        If None, current date will be used

    Returns
    -------
    log_file_path : Path
        The path to the log file
    """
    if log_file_dir is None:
        log_file_dir = get_log_file_directory()
    if name is None:
        name = time.strftime("%Y%m%d.log")
    log_file_path = log_file_dir.joinpath(name)

    return log_file_path


def get_bout_directory() -> Path:
    """
    Load the BOUT++ directory from the configuration file.

    Returns
    -------
    bout_path : Path
        Path to the BOUT++ repository
    """
    config = get_bout_runners_configuration()
    path_str = config["bout++"]["directory"]
    if "$HOME/" or "${HOME}/" in path_str.lower():
        path_str = "/".join(path_str.split("/")[1:])
        path_str = f"{Path.home()}/{path_str}"
    bout_path = Path(path_str).absolute()
    return bout_path



"""Contains unittests for the ProcessorSplit class."""


import pytest
from bout_runners.submitter.processor_split import ProcessorSplit


def test_processor_split() -> None:
    """Test that the ProcessorSplit is setting the parameters."""
    processor_split = ProcessorSplit(
        number_of_processors=1, number_of_nodes=1, processors_per_node=1
    )
    assert processor_split.number_of_processors == 1

    with pytest.raises(ValueError):
        processor_split.number_of_processors = 2



"""Module for configuring bout_runners."""


import logging
from pathlib import Path
from typing import Optional

import yaml
from bout_runners.utils.logs import get_log_config, set_up_logger
from bout_runners.utils.paths import (
    get_bout_directory,
    get_bout_log_config_path,
    get_bout_runners_config_path,
    get_bout_runners_configuration,
    get_log_file_directory,
)


def set_log_level(level: Optional[str] = None) -> None:
    """
    Set the log level.

    Parameters
    ----------
    level : None or str
        The logging level to use
        If None the caller will be prompted

    Raises
    ------
    ValueError
        If the level is not one of the possibilities
    """
    config = get_log_config()

    possibilities = ("DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL")

    if level is None:
        current_level = config["root"]["level"]

        possibilities_map = dict(enumerate(possibilities))
        question = (
            f"Please set the log level by entering a number:\n"
            f"   (empty input will reuse the current level "
            f"[{current_level}])\n"
        )
        for key, val in possibilities_map.items():
            question += f'{" "*3}({key}) - {val}\n'
        # Set an answer to start the wile loop
        answer_int = -1
        possibilities_keys = possibilities_map.keys()
        while answer_int not in possibilities_keys:
            answer_input = input(question)
            print(f"Your answered: '{answer_input}'")
            if answer_input != "":
                answer_int = int(answer_input)
            if answer_input == "":
                # Reverse the dict
                answer_int = list(possibilities_map.keys())[
                    list(possibilities_map.values()).index(current_level)
                ]
                break
        level = possibilities_map[answer_int]

    if level not in possibilities:
        msg = f"`level` in `set_log_level` must be one of " f"{possibilities}"
        raise ValueError(msg)

    print(f"Setting logging level to {level}")

    config["handlers"]["file_handler"]["level"] = level
    config["handlers"]["console_handler"]["level"] = level
    config["root"]["level"] = level

    with get_bout_log_config_path().open("w") as log_file:
        log_file.write(yaml.dump(config))

    set_up_logger(config)
    logging.debug("Logging level set to %s", level)
    print("")


def set_log_file_directory(log_dir: Optional[Path] = None) -> None:
    """
    Set the directory of the log files.

    Parameters
    ----------
    log_dir : None or Path
        The directory to keep the log files
        If None the caller will be prompted
    """
    config = get_bout_runners_configuration()
    if log_dir is None:
        current_dir = get_log_file_directory()
        question = (
            f"Please entering the directory for log files:\n"
            f"Empty input will reuse the current directory "
            f"{current_dir}\n"
        )
        answer = input(question)
        print(f"Your answered: '{answer}'")
        if answer == "":
            config["log"]["directory"] = str(current_dir)
        else:
            config["log"]["directory"] = answer
    else:
        config["log"]["directory"] = str(log_dir)

    with get_bout_runners_config_path().open("w") as configfile:
        config.write(configfile)

    print(f"Setting logging dir to {config['log']['directory']}")

    set_up_logger()
    logging.debug("Logging directory set to %s", config["log"]["directory"])
    print("")


def set_bout_directory(bout_dir: Optional[Path] = None) -> None:
    """
    Set the path to the BOUT++ directory.

    Parameters
    ----------
    bout_dir : None or Path
        The path to the BOUT++ directory
        If None, the caller will be prompted

    Raises
    ------
    ValueError
        If BOUT++ not found in the directory
    """
    config = get_bout_runners_configuration()
    if bout_dir is None:
        suggested_dir = get_bout_directory()
        question = (
            f"Please entering the directory for the root of BOUT++:\n"
            f"Empty input use the directory "
            f"{suggested_dir}\n"
        )
        answer = input(question)
        print(f"Your answered: '{answer}'")
        if answer != "":
            config["bout++"]["directory"] = answer
        else:
            config["bout++"]["directory"] = str(suggested_dir)
    else:
        config["bout++"]["directory"] = str(bout_dir)

    if not Path(config["bout++"]["directory"]).is_dir():
        msg = f'BOUT++ not found in {config["bout++"]["directory"]}'
        raise ValueError(msg)

    with get_bout_runners_config_path().open("w") as configfile:
        config.write(configfile)

    print(f"Setting BOUT++ directory to {config['bout++']['directory']}")

    set_up_logger()
    logging.debug("BOUT++ directory set to %s", config["bout++"]["directory"])
    print("")


def main() -> None:
    """Call all configuration functions."""
    set_log_level()
    set_log_file_directory()
    set_bout_directory()


if __name__ == "__main__":
    main()
